@@ -7,6 +7,8 @@
  * 	- max_alpha defaults to 65535 for RGB16/GREY16
  * 24/11/17 lovell
  * 	- match normalised alpha to output type
+ * 27/2/21 jjonesrs
+ * 	- revise range clipping and 1/x, again
  */
 
 /*
@@ -70,7 +72,8 @@ typedef VipsConversionClass VipsUnpremultiplyClass;
 
 G_DEFINE_TYPE( VipsUnpremultiply, vips_unpremultiply, VIPS_TYPE_CONVERSION );
 
-/* Unpremultiply an N-band image.
+/* Unpremultiply an N-band image. Don't use clip_alpha to calculate factor: we
+ * want over and undershoots on alpha and RGB to cancel.
  */
 #define UNPRE_MANY( IN, OUT ) { \
 	IN * restrict p = (IN *) in; \
@@ -78,18 +81,11 @@ G_DEFINE_TYPE( VipsUnpremultiply, vips_unpremultiply, VIPS_TYPE_CONVERSION );
 	\
 	for( x = 0; x < width; x++ ) { \
 		IN alpha = p[alpha_band]; \
+		OUT factor = alpha == 0 ? 0 : max_alpha / alpha; \
 		\
-		if( alpha != 0 ) { \
-			OUT factor = max_alpha / alpha; \
-			\
-			for( i = 0; i < alpha_band; i++ ) \
-				q[i] = factor * p[i]; \
-			q[alpha_band] = alpha; \
-		} \
-		else \
-			for( i = 0; i < alpha_band + 1; i++ ) \
-				q[i] = 0; \
-		\
+		for( i = 0; i < alpha_band; i++ ) \
+			q[i] = factor * p[i]; \
+		q[alpha_band] = VIPS_CLIP( 0, alpha, max_alpha ); \
 		for( i = alpha_band + 1; i < bands; i++ ) \
 			q[i] = p[i]; \
 		\
@@ -106,21 +102,12 @@ G_DEFINE_TYPE( VipsUnpremultiply, vips_unpremultiply, VIPS_TYPE_CONVERSION );
 	\
 	for( x = 0; x < width; x++ ) { \
 		IN alpha = p[3]; \
+		OUT factor = alpha == 0 ? 0 : max_alpha / alpha; \
 		\
-		if( alpha != 0 ) { \
-			OUT factor = max_alpha / alpha; \
-			\
-			q[0] = factor * p[0]; \
-			q[1] = factor * p[1]; \
-			q[2] = factor * p[2]; \
-			q[3] = alpha; \
-		} \
-		else { \
-			q[0] = 0; \
-			q[1] = 0; \
-			q[2] = 0; \
-			q[3] = 0; \
-		} \
+		q[0] = factor * p[0]; \
+		q[1] = factor * p[1]; \
+		q[2] = factor * p[2]; \
+		q[3] = VIPS_CLIP( 0, alpha, max_alpha ); \
 		\
 		p += 4; \
 		q += 4; \
