@@ -277,8 +277,6 @@ vips_worley_build( VipsObject *object )
 		VIPS_ROUND_UP( worley->height, worley->cell_size ) / 
 		worley->cell_size;
 
-	worley->seed = g_random_double() * 0xffffffffu;
-
 	vips_image_init_fields( create->out,
 		worley->width, worley->height, 1,
 		VIPS_FORMAT_FLOAT, VIPS_CODING_NONE, 
@@ -328,12 +326,20 @@ vips_worley_class_init( VipsWorleyClass *class )
 		G_STRUCT_OFFSET( VipsWorley, cell_size ),
 		1, VIPS_MAX_COORD, 256 );
 
+	VIPS_ARG_INT( class, "seed", 4, 
+		_( "Seed" ), 
+		_( "Random number seed" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsWorley, seed ),
+		INT_MIN, INT_MAX, 0 );
+
 }
 
 static void
 vips_worley_init( VipsWorley *worley )
 {
 	worley->cell_size = 256;
+	worley->seed = UINT_MAX * g_random_double();
 }
 
 /**
