@@ -173,7 +173,13 @@
 
 #ifdef HAVE_GSF
 
+/* Disable deprecation warnings from gsf. There are loads, and still not
+ * patched as of 12/2020.
+ */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include <gsf/gsf.h>
+#pragma GCC diagnostic pop
 
 /* Simple wrapper around libgsf.
  *
@@ -360,7 +366,8 @@ vips_gsf_path( VipsGsfDirectory *tree, const char *name, ... )
 	 * path we are creating.
 	 */
 	tree->file_count += 1;
-	tree->filename_lengths += strlen( tree->out->name ) + strlen( name ) + 1;
+	tree->filename_lengths += 
+		strlen( tree->out->name ) + strlen( name ) + 1;
 
 	dir = tree; 
 	va_start( ap, name );
