@@ -2,6 +2,9 @@
  *
  * 29/6/18
  * 	- from fitsload.c
+ * 9/9/19
+ * 	- use double for all floating point scalar metadata, like other loaders
+ * 	- remove stray use of "n" property
  */
 
 /*
@@ -71,9 +74,13 @@
 typedef struct _VipsForeignLoadNifti {
 	VipsForeignLoad parent_object;
 
-	/* Filename for load.
+	/* Source to load from (set by subclasses).
 	 */
-	char *filename; 
+	VipsSource *source;
+
+	/* Filename from source.
+	 */
+	const char *filename;
 
 	/* The NIFTI image loaded to memory.
 	 */
@@ -88,7 +95,7 @@ typedef struct _VipsForeignLoadNifti {
 
 typedef VipsForeignLoadClass VipsForeignLoadNiftiClass;
 
-G_DEFINE_TYPE( VipsForeignLoadNifti, vips_foreign_load_nifti, 
+G_DEFINE_ABSTRACT_TYPE( VipsForeignLoadNifti, vips_foreign_load_nifti, 
 	VIPS_TYPE_FOREIGN_LOAD );
 
 static void
@@ -96,6 +103,7 @@ vips_foreign_load_nifti_dispose( GObject *gobject )
 {
 	VipsForeignLoadNifti *nifti = (VipsForeignLoadNifti *) gobject;
 
+	VIPS_UNREF( nifti->source );
 	VIPS_UNREF( nifti->memory );
 	VIPS_FREEF( nifti_image_free, nifti->nim );
 
@@ -104,38 +112,27 @@ vips_foreign_load_nifti_dispose( GObject *gobject )
 }
 
 static int
-vips_foreign_load_nifti_is_a( const char *filename )
+vips_foreign_load_nifti_build( VipsObject *object )
 {
-	char *hfile;
-	znzFile fp;
-	nifti_1_header nhdr;
+	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
+	VipsForeignLoadNifti *nifti = (VipsForeignLoadNifti *) object;
 
-	/* Unfortunately is_nifti_file() is very slow and produces lots of
-	 * output. We have to make our own.
+	/* We can only open source which have an associated filename, since
+	 * the nifti library works in terms of filenames.
 	 */
-
-	if( !(hfile = nifti_findhdrname( filename )) )
-		return( 0 );
-
-	fp = znzopen( hfile, "rb", nifti_is_gzfile( hfile ));
-	if( znz_isnull( fp ) ) { 
-		free( hfile );
-		return( 0 );
+	if( nifti->source ) {
+		nifti->filename = vips_connection_filename( VIPS_CONNECTION( 
+			nifti->source ) );
+		if( !nifti->filename ) {
+			vips_error( class->nickname, "%s", 
+				_( "no filename available" ) );
+			return( -1 );
+		}
 	}
-	free( hfile );
 
-	(void) znzread( &nhdr, 1, sizeof( nhdr ), fp );
-
-	znzclose( fp );
-
-	/* Test for sanity both ways around. There's a thing to test for byte
-	 * order in niftilib, but it's static :(
-	 */
-	if( nifti_hdr_looks_good( &nhdr ) ) 
-		return( 1 );
-	swap_nifti_header( &nhdr, FALSE );
-	if( nifti_hdr_looks_good( &nhdr ) ) 
-		return( 1 );
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_nifti_parent_class )->
+		build( object ) )
+		return( -1 );
 
 	return( 0 );
 }
@@ -207,22 +204,22 @@ static VipsForeignNiftiFields vips_foreign_nifti_fields[] = {
 	{ "nv", G_TYPE_INT, G_STRUCT_OFFSET( nifti_image, nv ) }, 
 	{ "nw", G_TYPE_INT, G_STRUCT_OFFSET( nifti_image, nw ) }, 
 
-	{ "dx", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dx ) }, 
-	{ "dy", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dy ) }, 
-	{ "dz", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dz ) }, 
-	{ "dt", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dt ) }, 
-	{ "du", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, du ) }, 
-	{ "dv", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dv ) }, 
-	{ "dw", G_TYPE_FLOAT, G_STRUCT_OFFSET( nifti_image, dw ) }, 
+	{ "dx", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dx ) }, 
+	{ "dy", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dy ) }, 
+	{ "dz", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dz ) }, 
+	{ "dt", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dt ) }, 
+	{ "du", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, du ) }, 
+	{ "dv", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dv ) }, 
+	{ "dw", G_TYPE_DOUBLE, G_STRUCT_OFFSET( nifti_image, dw ) }, 
 
-	{ "scl_slope", G_TYPE_FLOAT, 
+	{ "scl_slope", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, scl_slope ) }, 
-	{ "scl_inter", G_TYPE_FLOAT, 
+	{ "scl_inter", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, scl_inter ) }, 
 
-	{ "cal_min", G_TYPE_FLOAT, 
+	{ "cal_min", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, cal_min ) }, 
-	{ "cal_max", G_TYPE_FLOAT, 
+	{ "cal_max", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, cal_max ) }, 
 
 	{ "qform_code", G_TYPE_INT, 
@@ -243,61 +240,61 @@ static VipsForeignNiftiFields vips_foreign_nifti_fields[] = {
 		G_STRUCT_OFFSET( nifti_image, slice_start ) }, 
 	{ "slice_end", G_TYPE_INT, 
 		G_STRUCT_OFFSET( nifti_image, slice_end ) }, 
-	{ "slice_duration", G_TYPE_FLOAT, 
+	{ "slice_duration", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, slice_duration ) }, 
 
-	{ "quatern_b", G_TYPE_FLOAT, 
+	{ "quatern_b", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, quatern_b ) }, 
-	{ "quatern_c", G_TYPE_FLOAT, 
+	{ "quatern_c", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, quatern_c ) }, 
-	{ "quatern_d", G_TYPE_FLOAT, 
+	{ "quatern_d", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, quatern_d ) }, 
-	{ "qoffset_x", G_TYPE_FLOAT, 
+	{ "qoffset_x", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, qoffset_x ) }, 
-	{ "qoffset_y", G_TYPE_FLOAT, 
+	{ "qoffset_y", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, qoffset_y ) }, 
-	{ "qoffset_z", G_TYPE_FLOAT, 
+	{ "qoffset_z", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, qoffset_z ) }, 
-	{ "qfac", G_TYPE_FLOAT, 
+	{ "qfac", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, qfac ) }, 
 
-	{ "sto_xyz00", G_TYPE_FLOAT, 
+	{ "sto_xyz00", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[0][0] ) }, 
-	{ "sto_xyz01", G_TYPE_FLOAT, 
+	{ "sto_xyz01", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[0][1] ) }, 
-	{ "sto_xyz02", G_TYPE_FLOAT, 
+	{ "sto_xyz02", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[0][2] ) }, 
-	{ "sto_xyz03", G_TYPE_FLOAT, 
+	{ "sto_xyz03", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[0][3] ) }, 
 
-	{ "sto_xyz10", G_TYPE_FLOAT, 
+	{ "sto_xyz10", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[1][0] ) }, 
-	{ "sto_xyz11", G_TYPE_FLOAT, 
+	{ "sto_xyz11", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[1][1] ) }, 
-	{ "sto_xyz12", G_TYPE_FLOAT, 
+	{ "sto_xyz12", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[1][2] ) }, 
-	{ "sto_xyz13", G_TYPE_FLOAT, 
+	{ "sto_xyz13", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[1][3] ) }, 
 
-	{ "sto_xyz20", G_TYPE_FLOAT, 
+	{ "sto_xyz20", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[2][0] ) }, 
-	{ "sto_xyz21", G_TYPE_FLOAT, 
+	{ "sto_xyz21", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[2][1] ) }, 
-	{ "sto_xyz22", G_TYPE_FLOAT, 
+	{ "sto_xyz22", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[2][2] ) }, 
-	{ "sto_xyz23", G_TYPE_FLOAT, 
+	{ "sto_xyz23", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[2][3] ) }, 
 
-	{ "sto_xyz30", G_TYPE_FLOAT, 
+	{ "sto_xyz30", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[3][0] ) }, 
-	{ "sto_xyz31", G_TYPE_FLOAT, 
+	{ "sto_xyz31", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[3][1] ) }, 
-	{ "sto_xyz32", G_TYPE_FLOAT, 
+	{ "sto_xyz32", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[3][2] ) }, 
-	{ "sto_xyz33", G_TYPE_FLOAT, 
+	{ "sto_xyz33", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, sto_xyz.m[3][3] ) }, 
 
-	{ "toffset", G_TYPE_FLOAT, 
+	{ "toffset", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, toffset ) }, 
 
 	{ "xyz_units", G_TYPE_INT, 
@@ -309,11 +306,11 @@ static VipsForeignNiftiFields vips_foreign_nifti_fields[] = {
 		G_STRUCT_OFFSET( nifti_image, nifti_type ) }, 
 	{ "intent_code", G_TYPE_INT, 
 		G_STRUCT_OFFSET( nifti_image, intent_code ) }, 
-	{ "intent_p1", G_TYPE_FLOAT, 
+	{ "intent_p1", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, intent_p1 ) }, 
-	{ "intent_p2", G_TYPE_FLOAT, 
+	{ "intent_p2", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, intent_p2 ) }, 
-	{ "intent_p3", G_TYPE_FLOAT, 
+	{ "intent_p3", G_TYPE_DOUBLE, 
 		G_STRUCT_OFFSET( nifti_image, intent_p3 ) }, 
 };
 
@@ -349,8 +346,11 @@ vips_gvalue_read( GValue *value, void *p )
 		g_value_set_int( value, *((int *) p) );
 		break;
 
-	case G_TYPE_FLOAT:
-		g_value_set_float( value, *((float *) p) );
+	case G_TYPE_DOUBLE:
+		/* We set as double rather than float, as things like pyvips
+		 * expect double for metadata items.
+		 */
+		g_value_set_double( value, *((float *) p) );
 		break;
 
 	default:
@@ -505,8 +505,7 @@ vips_foreign_load_nifti_set_header( VipsForeignLoadNifti *nifti,
 		vips_image_set_blob_copy( out, txt, ext->edata, ext->esize );
 	}
 
-	if( vips_object_argument_isset( VIPS_OBJECT( nifti ), "n" ) )
-		vips_image_set_int( out, VIPS_META_PAGE_HEIGHT, nim->ny );
+	vips_image_set_int( out, VIPS_META_PAGE_HEIGHT, nim->ny );
 
 	return( 0 );
 }
@@ -571,14 +570,6 @@ vips_foreign_load_nifti_load( VipsForeignLoad *load )
 	return( 0 );
 }
 
-const char *vips__nifti_suffs[] = { 
-	".nii", ".nii.gz", 
-	".hdr", ".hdr.gz", 
-	".img", ".img.gz", 
-	".nia", ".nia.gz", 
-	NULL 
-};
-
 static void
 vips_foreign_load_nifti_class_init( VipsForeignLoadNiftiClass *class )
 {
@@ -591,29 +582,208 @@ vips_foreign_load_nifti_class_init( VipsForeignLoadNiftiClass *class )
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
 
-	object_class->nickname = "niftiload";
+	object_class->nickname = "niftiload_base";
 	object_class->description = _( "load a NIFTI image" );
+	object_class->build = vips_foreign_load_nifti_build;
 
 	/* is_a() is not that quick ... lower the priority.
 	 */
 	foreign_class->priority = -50;
 
-	foreign_class->suffs = vips__nifti_suffs;
-
-	load_class->is_a = vips_foreign_load_nifti_is_a;
 	load_class->header = vips_foreign_load_nifti_header;
 	load_class->load = vips_foreign_load_nifti_load;
+}
+
+static void
+vips_foreign_load_nifti_init( VipsForeignLoadNifti *nifti )
+{
+}
+
+typedef struct _VipsForeignLoadNiftiFile {
+	VipsForeignLoadNifti parent_object;
+
+	/* Filename for load.
+	 */
+	char *filename; 
+
+} VipsForeignLoadNiftiFile;
+
+typedef VipsForeignLoadNiftiClass VipsForeignLoadNiftiFileClass;
+
+G_DEFINE_TYPE( VipsForeignLoadNiftiFile, vips_foreign_load_nifti_file, 
+	vips_foreign_load_nifti_get_type() );
+
+static int
+vips_foreign_load_nifti_file_build( VipsObject *object )
+{
+	VipsForeignLoadNifti *nifti = (VipsForeignLoadNifti *) object;
+	VipsForeignLoadNiftiFile *file = (VipsForeignLoadNiftiFile *) object;
+
+	if( file->filename &&
+		!(nifti->source = 
+			vips_source_new_from_file( file->filename )) )
+		return( -1 );
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_nifti_file_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+const char *vips_foreign_nifti_suffs[] = { 
+	".nii", ".nii.gz", 
+	".hdr", ".hdr.gz", 
+	".img", ".img.gz", 
+	".nia", ".nia.gz", 
+	NULL 
+};
+
+static int
+vips_foreign_load_nifti_is_a( const char *filename )
+{
+	char *hfile;
+	znzFile fp;
+	nifti_1_header nhdr;
+
+	/* Unfortunately is_nifti_file() is very slow and produces lots of
+	 * output. We have to make our own.
+	 */
+
+	if( !(hfile = nifti_findhdrname( filename )) )
+		return( 0 );
+
+	fp = znzopen( hfile, "rb", nifti_is_gzfile( hfile ));
+	if( znz_isnull( fp ) ) { 
+		free( hfile );
+		return( 0 );
+	}
+	free( hfile );
+
+	(void) znzread( &nhdr, 1, sizeof( nhdr ), fp );
+
+	znzclose( fp );
+
+	/* Test for sanity both ways around. There's a thing to test for byte
+	 * order in niftilib, but it's static :(
+	 */
+	if( nifti_hdr_looks_good( &nhdr ) ) 
+		return( 1 );
+	swap_nifti_header( &nhdr, FALSE );
+	if( nifti_hdr_looks_good( &nhdr ) ) 
+		return( 1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_load_nifti_file_class_init( 
+	VipsForeignLoadNiftiFileClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
+	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "niftiload";
+	object_class->description = _( "load NIfTI volume" );
+	object_class->build = vips_foreign_load_nifti_file_build;
+
+	foreign_class->suffs = vips_foreign_nifti_suffs;
+
+	load_class->is_a = vips_foreign_load_nifti_is_a;
 
 	VIPS_ARG_STRING( class, "filename", 1, 
 		_( "Filename" ),
 		_( "Filename to load from" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT, 
-		G_STRUCT_OFFSET( VipsForeignLoadNifti, filename ),
+		G_STRUCT_OFFSET( VipsForeignLoadNiftiFile, filename ),
 		NULL );
+
 }
 
 static void
-vips_foreign_load_nifti_init( VipsForeignLoadNifti *nifti )
+vips_foreign_load_nifti_file_init( VipsForeignLoadNiftiFile *nifti )
+{
+}
+
+typedef struct _VipsForeignLoadNiftiSource {
+	VipsForeignLoadNifti parent_object;
+
+	/* Load from a source.
+	 */
+	VipsSource *source;
+
+} VipsForeignLoadNiftiSource;
+
+typedef VipsForeignLoadNiftiClass VipsForeignLoadNiftiSourceClass;
+
+G_DEFINE_TYPE( VipsForeignLoadNiftiSource, vips_foreign_load_nifti_source, 
+	vips_foreign_load_nifti_get_type() );
+
+static int
+vips_foreign_load_nifti_source_build( VipsObject *object )
+{
+	VipsForeignLoadNifti *nifti = (VipsForeignLoadNifti *) object;
+	VipsForeignLoadNiftiSource *source = 
+		(VipsForeignLoadNiftiSource *) object;
+
+	if( source->source ) {
+		nifti->source = source->source;
+		g_object_ref( nifti->source );
+	}
+
+	if( VIPS_OBJECT_CLASS( 
+		vips_foreign_load_nifti_source_parent_class )->
+			build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static gboolean
+vips_foreign_load_nifti_source_is_a_source( VipsSource *source )
+{
+	const char *filename;
+
+	return( (filename = 
+		vips_connection_filename( VIPS_CONNECTION( source ) )) &&
+		vips_foreign_load_nifti_is_a( filename ) );
+}
+
+static void
+vips_foreign_load_nifti_source_class_init( 
+	VipsForeignLoadNiftiSourceClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "niftiload_source";
+	object_class->description = _( "load NIfTI volumes" );
+	object_class->build = vips_foreign_load_nifti_source_build;
+
+	load_class->is_a_source = 
+		vips_foreign_load_nifti_source_is_a_source;
+
+	VIPS_ARG_OBJECT( class, "source", 1,
+		_( "Source" ),
+		_( "Source to load from" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignLoadNiftiSource, source ),
+		VIPS_TYPE_SOURCE );
+
+}
+
+static void
+vips_foreign_load_nifti_source_init( 
+	VipsForeignLoadNiftiSource *nifti )
 {
 }
 
@@ -645,3 +815,26 @@ vips_niftiload( const char *filename, VipsImage **out, ... )
 
 	return( result );
 }
+
+/**
+ * vips_niftiload_source:
+ * @source: source to load from
+ * @out: (out): decompressed image
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Exactly as vips_niftiload(), but read from a source. 
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_niftiload_source( VipsSource *source, VipsImage **out, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, out );
+	result = vips_call_split( "niftiload_source", ap, source, out );
+	va_end( ap );
+
+	return( result );
+}
