@@ -45,6 +45,10 @@ extern "C" {
 )
 #endif /*HAVE_CHECKED_MUL*/
 
+/* We've seen real images with 28 chunks, so set 50.
+ */
+#define MAX_PNG_TEXT_CHUNKS 50
+
 void vips__tiff_init( void );
 
 int vips__tiff_write( VipsImage *in, const char *filename, 
@@ -177,9 +181,9 @@ int vips__jpeg_read_source( VipsSource *source, VipsImage *out,
 int vips__isjpeg_source( VipsSource *source );
 
 int vips__png_ispng_source( VipsSource *source );
-int vips__png_header_source( VipsSource *source, VipsImage *out );
+int vips__png_header_source( VipsSource *source, VipsImage *out, gboolean unlimited );
 int vips__png_read_source( VipsSource *source, VipsImage *out, 
-	gboolean fail );
+	gboolean fail, gboolean unlimited );
 gboolean vips__png_isinterlaced_source( VipsSource *source );
 extern const char *vips__png_suffs[];
 
