@@ -10,6 +10,8 @@
  * 	- add ppmsave_target
  * 20/11/20
  * 	- byteswap on save, if necessary [ewelot]
+ * 2/12/20
+ * 	- don't add date with @strip [ewelot]
  */
 
 /*
@@ -210,11 +212,50 @@ vips_foreign_save_ppm_block( VipsRegion *region, VipsRect *area, void *a )
 }
 
 static int
-vips_foreign_save_ppm( VipsForeignSavePpm *ppm, VipsImage *image )
+vips_foreign_save_ppm_build( VipsObject *object )
 {
+	VipsForeignSave *save = (VipsForeignSave *) object;
+	VipsForeignSavePpm *ppm = (VipsForeignSavePpm *) object;
+
+	VipsImage *image;
 	char *magic;
 	char *date;
 
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_ppm_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	image = save->ready;
+
+        /* Handle the deprecated squash parameter.
+	 */
+        if( vips_object_argument_isset( object, "squash" ) ) 
+		ppm->bitdepth = 1;
+
+	if( vips_check_uintorf( "vips2ppm", image ) || 
+		vips_check_bands_1or3( "vips2ppm", image ) || 
+		vips_check_uncoded( "vips2ppm", image ) || 
+		vips_image_pio_input( image ) )
+		return( -1 );
+
+	if( ppm->ascii && 
+		image->BandFmt == VIPS_FORMAT_FLOAT ) {
+		g_warning( "%s", 
+			_( "float images must be binary -- disabling ascii" ) );
+		ppm->ascii = FALSE;
+	}
+
+	/* One bit images must come from a 8 bit, one band source. 
+	 */
+	if( ppm->bitdepth && 
+		(image->Bands != 1 || 
+		 image->BandFmt != VIPS_FORMAT_UCHAR) ) {
+		g_warning( "%s", 
+			_( "can only save 1 band uchar images as 1 bit -- " 
+				"disabling 1 bit save" ) );
+		ppm->bitdepth = 0; 
+	}
+
 	magic = "unset";
 	if( image->BandFmt == VIPS_FORMAT_FLOAT && 
 		image->Bands == 3 ) 
@@ -246,10 +287,12 @@ vips_foreign_save_ppm( VipsForeignSavePpm *ppm, VipsImage *image )
 		g_assert_not_reached();
 
 	vips_target_writef( ppm->target, "%s\n", magic );
-	date = vips__get_iso8601();
-	vips_target_writef( ppm->target, 
-		"#vips2ppm - %s\n", date );
-	g_free( date );
+	if( !save->strip ) {
+		date = vips__get_iso8601();
+		vips_target_writef( ppm->target, 
+			"#vips2ppm - %s\n", date );
+		g_free( date );
+	}
 	vips_target_writef( ppm->target, 
 		"%d %d\n", image->Xsize, image->Ysize );
 
@@ -284,6 +327,7 @@ vips_foreign_save_ppm( VipsForeignSavePpm *ppm, VipsImage *image )
 			 */
 			g_ascii_dtostr( buf, G_ASCII_DTOSTR_BUF_SIZE, scale );
 			vips_target_writes( ppm->target, buf );
+			vips_target_writes( ppm->target, "\n" );
 }
 			break;
 
@@ -309,8 +353,11 @@ vips_foreign_save_ppm( VipsForeignSavePpm *ppm, VipsImage *image )
 
 		if( vips__byteswap_bool( image, &x, !vips_amiMSBfirst() ) )
 			return( -1 );
-		VIPS_UNREF( image );
 		image = x;
+
+		/* image must now be unreffed on exit.
+		 */
+		vips_object_local( VIPS_OBJECT( ppm->target ), image );
 	}
 
 	if( vips_sink_disc( image, vips_foreign_save_ppm_block, ppm ) )
@@ -321,54 +368,6 @@ vips_foreign_save_ppm( VipsForeignSavePpm *ppm, VipsImage *image )
 	return( 0 );
 }
 
-static int
-vips_foreign_save_ppm_build( VipsObject *object )
-{
-	VipsForeignSave *save = (VipsForeignSave *) object;
-	VipsForeignSavePpm *ppm = (VipsForeignSavePpm *) object;
-
-	VipsImage *image;
-
-	if( VIPS_OBJECT_CLASS( vips_foreign_save_ppm_parent_class )->
-		build( object ) )
-		return( -1 );
-
-        /* Handle the deprecated squash parameter.
-	 */
-        if( vips_object_argument_isset( object, "squash" ) ) 
-		ppm->bitdepth = 1;
-
-	image = save->ready;
-	if( vips_check_uintorf( "vips2ppm", image ) || 
-		vips_check_bands_1or3( "vips2ppm", image ) || 
-		vips_check_uncoded( "vips2ppm", image ) || 
-		vips_image_pio_input( image ) )
-		return( -1 );
-
-	if( ppm->ascii && 
-		image->BandFmt == VIPS_FORMAT_FLOAT ) {
-		g_warning( "%s", 
-			_( "float images must be binary -- disabling ascii" ) );
-		ppm->ascii = FALSE;
-	}
-
-	/* One bit images must come from a 8 bit, one band source. 
-	 */
-	if( ppm->bitdepth && 
-		(image->Bands != 1 || 
-		 image->BandFmt != VIPS_FORMAT_UCHAR) ) {
-		g_warning( "%s", 
-			_( "can only save 1 band uchar images as 1 bit -- " 
-				"disabling 1 bit save" ) );
-		ppm->bitdepth = 0; 
-	}
-
-	if( vips_foreign_save_ppm( ppm, image ) )
-		return( -1 );
-
-	return( 0 );
-}
-
 /* Save a bit of typing.
  */
 #define UC VIPS_FORMAT_UCHAR
