@@ -245,9 +245,11 @@ vips_foreign_load_svg_get_flags( VipsForeignLoad *load )
 	return( VIPS_FOREIGN_SEQUENTIAL );
 }
 
-static void
+static int
 vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 {
+	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( svg );
+
 	RsvgDimensionData dimensions;
 	int width;
 	int height;
@@ -261,6 +263,11 @@ vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 	width = dimensions.width;
 	height = dimensions.height;
 
+	if( width <= 0 || height <= 0 ) {
+		vips_error( class->nickname, "%s", _( "bad dimensions" ) );
+		return( -1 );
+	}
+
 	/* Calculate dimensions at required dpi/scale.
 	 */
 	scale = svg->scale * svg->dpi / 72.0;
@@ -300,6 +307,7 @@ vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 	 */
         vips_image_pipelinev( out, VIPS_DEMAND_STYLE_FATSTRIP, NULL );
 
+	return( 0 );
 }
 
 static int
@@ -307,9 +315,7 @@ vips_foreign_load_svg_header( VipsForeignLoad *load )
 {
 	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
 
-	vips_foreign_load_svg_parse( svg, load->out ); 
-
-	return( 0 );
+	return vips_foreign_load_svg_parse( svg, load->out );
 }
 
 static int
@@ -350,6 +356,7 @@ vips_foreign_load_svg_generate( VipsRegion *or,
 	 * running inside a non-threaded tilecache.
 	 */
 	if( !rsvg_handle_render_cairo( svg->page, cr ) ) {
+		cairo_destroy( cr );
 		vips_operation_invalidate( VIPS_OPERATION( svg ) );
 		vips_error( class->nickname, 
 			"%s", _( "SVG rendering failed" ) );
@@ -361,7 +368,7 @@ vips_foreign_load_svg_generate( VipsRegion *or,
 	/* Cairo makes pre-multipled BRGA -- we must byteswap and unpremultiply.
 	 */
 	for( y = 0; y < r->height; y++ ) 
-		vips__cairo2rgba( 
+                vips__premultiplied_bgra2rgba( 
 			(guint32 *) VIPS_REGION_ADDR( or, r->left, r->top + y ),
 			r->width ); 
 
@@ -385,9 +392,9 @@ vips_foreign_load_svg_load( VipsForeignLoad *load )
 	 * Make tiles 2000 pixels high to limit overcomputation. 
 	 */
 	t[0] = vips_image_new(); 
-	vips_foreign_load_svg_parse( svg, t[0] ); 
-	if( vips_image_generate( t[0], 
-		NULL, vips_foreign_load_svg_generate, NULL, svg, NULL ) ||
+	if( vips_foreign_load_svg_parse( svg, t[0] ) ||
+		vips_image_generate( t[0], NULL,
+			vips_foreign_load_svg_generate, NULL, svg, NULL ) ||
 		vips_tilecache( t[0], &t[1],
 			"tile_width", VIPS_MIN( t[0]->Xsize, RSVG_MAX_WIDTH ),
 			"tile_height", 2000,
