@@ -270,6 +270,16 @@
  */
 #define MAX_ALPHA (64)
 
+/* Bioformats uses this tag for lossy jp2k compressed tiles.
+ */
+#define JP2K_LOSSY 33004
+
+/* Compression types we handle ourselves.
+ */
+static int wtiff_we_compress[] = {
+	JP2K_LOSSY
+};
+
 typedef struct _Layer Layer;
 typedef struct _Wtiff Wtiff;
 
@@ -351,9 +361,10 @@ struct _Wtiff {
 	int strip;			/* Don't write metadata */
 	VipsRegionShrink region_shrink; /* How to shrink regions */
 	int level;			/* zstd compression level */
-	gboolean lossless;		/* webp lossless mode */
+	gboolean lossless;		/* lossless mode */
 	VipsForeignDzDepth depth;	/* Pyr depth */
 	gboolean subifd;		/* Write pyr layers into subifds */
+	gboolean premultiply;		/* Premultiply alpha */
 
 	/* True if we've detected a toilet-roll image, plus the page height,
 	 * which has been checked to be a factor of im->Ysize. page_number
@@ -368,6 +379,11 @@ struct _Wtiff {
 	 * roll mode.
 	 */
 	int image_height;
+
+	/* TRUE if the compression type is not supported by libtiff directly
+	 * and we must compress ourselves. 
+	 */
+	gboolean we_compress;
 };
 
 /* Write an ICC Profile from a file into the JPEG stream.
@@ -636,6 +652,7 @@ wtiff_write_header( Wtiff *wtiff, Layer *layer )
 {
 	TIFF *tif = layer->tif;
 
+	int i;
 	int orientation; 
 
 #ifdef DEBUG
@@ -672,6 +689,12 @@ wtiff_write_header( Wtiff *wtiff, Layer *layer )
 		wtiff->predictor != VIPS_FOREIGN_TIFF_PREDICTOR_NONE ) 
 		TIFFSetField( tif, TIFFTAG_PREDICTOR, wtiff->predictor );
 
+	for( i = 0; i < VIPS_NUMBER( wtiff_we_compress ); i++ )
+		if( wtiff->compression == wtiff_we_compress[i] ) {
+			wtiff->we_compress = TRUE;
+			break;
+		}
+
 	/* Don't write mad resolutions (eg. zero), it confuses some programs.
 	 */
 	TIFFSetField( tif, TIFFTAG_RESOLUTIONUNIT, wtiff->resunit );
@@ -791,9 +814,14 @@ wtiff_write_header( Wtiff *wtiff, Layer *layer )
 			/* EXTRASAMPLE_UNASSALPHA means generic extra
 			 * alpha-like channels. ASSOCALPHA means pre-multipled
 			 * alpha only. 
+			 *
+			 * Make the first channel the premultiplied alpha, if
+			 * we are premultiplying.
 			 */
 			for( i = 0; i < alpha_bands; i++ )
-				v[i] = EXTRASAMPLE_UNASSALPHA;
+				v[i] = i == 0 && wtiff->premultiply ? 
+					EXTRASAMPLE_ASSOCALPHA :
+					EXTRASAMPLE_UNASSALPHA;
 			TIFFSetField( tif, 
 				TIFFTAG_EXTRASAMPLES, alpha_bands, v );
 		}
@@ -981,7 +1009,6 @@ static void
 wtiff_free( Wtiff *wtiff )
 {
 	wtiff_delete_temps( wtiff );
-
 	VIPS_UNREF( wtiff->ready );
 	VIPS_FREE( wtiff->tbuf );
 	VIPS_FREEF( layer_free_all, wtiff->layer );
@@ -1011,6 +1038,8 @@ get_compression( VipsForeignTiffCompression compression )
 	case VIPS_FOREIGN_TIFF_COMPRESSION_ZSTD:
 		return( COMPRESSION_ZSTD );
 #endif /*HAVE_TIFF_COMPRESSION_WEBP*/
+	case VIPS_FOREIGN_TIFF_COMPRESSION_JP2K:
+		return( JP2K_LOSSY );
 	
 	default:
 		return( COMPRESSION_NONE );
@@ -1040,24 +1069,56 @@ get_resunit( VipsForeignTiffResunit resunit )
 static int
 ready_to_write( Wtiff *wtiff )
 {
-	if( vips_check_coding_known( "vips2tiff", wtiff->input ) )
+	VipsImage *input;
+	VipsImage *x;
+
+	input = wtiff->input;
+	g_object_ref( input );
+
+	if( vips_check_coding_known( "vips2tiff", input ) ) {
+		VIPS_UNREF( input );
 		return( -1 );
+	}
+
+	/* Premultiply any alpha, if necessary.
+	 */
+	if( wtiff->premultiply &&
+		vips_image_hasalpha( input ) ) {
+		VipsBandFormat start_format = input->BandFmt;
+
+		if( vips_premultiply( input, &x, NULL ) ) {
+			VIPS_UNREF( input );
+			return( -1 );
+		}
+		VIPS_UNREF( input );
+		input = x;
+
+		/* Premultiply always makes a float -- cast back again.
+		 */
+		if( vips_cast( input, &x, start_format, NULL ) ) {
+			VIPS_UNREF( input );
+			return( -1 );
+		}
+		VIPS_UNREF( input );
+		input = x;
+	}
 
 	/* "squash" float LAB down to LABQ.
 	 */
 	if( wtiff->bitdepth &&
-		wtiff->input->Bands == 3 &&
-		wtiff->input->BandFmt == VIPS_FORMAT_FLOAT &&
-		wtiff->input->Type == VIPS_INTERPRETATION_LAB ) {
-		if( vips_Lab2LabQ( wtiff->input, &wtiff->ready, NULL ) )
+		input->Bands == 3 &&
+		input->BandFmt == VIPS_FORMAT_FLOAT &&
+		input->Type == VIPS_INTERPRETATION_LAB ) {
+		if( vips_Lab2LabQ( input, &x, NULL ) ) {
+			VIPS_UNREF( input );
 			return( -1 );
-		wtiff->bitdepth = 0;
-	}
-	else {
-		wtiff->ready = wtiff->input;
-		g_object_ref( wtiff->ready );
+		}
+		VIPS_UNREF( input );
+		input = x;
 	}
 
+	wtiff->ready = input;
+
 	return( 0 );
 }
 
@@ -1079,7 +1140,8 @@ wtiff_new( VipsImage *input, const char *filename,
 	int level, 
 	gboolean lossless,
 	VipsForeignDzDepth depth, 
-	gboolean subifd )
+	gboolean subifd,
+	gboolean premultiply )
 {
 	Wtiff *wtiff;
 
@@ -1112,6 +1174,7 @@ wtiff_new( VipsImage *input, const char *filename,
 	wtiff->lossless = lossless;
 	wtiff->depth = depth;
 	wtiff->subifd = subifd;
+	wtiff->premultiply = premultiply;
 	wtiff->toilet_roll = FALSE;
 	wtiff->page_height = vips_image_get_page_height( input );
 	wtiff->page_number = 0;
@@ -1237,21 +1300,6 @@ wtiff_new( VipsImage *input, const char *filename,
 		wtiff->miniswhite = FALSE;
 	}
 
-	/* lossless is for webp only.
-	 */
-#ifdef HAVE_TIFF_COMPRESSION_WEBP
-	if( wtiff->lossless ) {
-		if( wtiff->compression == COMPRESSION_NONE )
-			wtiff->compression = COMPRESSION_WEBP;
-
-		if( wtiff->compression != COMPRESSION_WEBP ) {
-			g_warning( "%s", 
-				_( "lossless is for WEBP compression only" ) );
-			wtiff->lossless = FALSE;
-		}
-	}
-#endif /*HAVE_TIFF_COMPRESSION_WEBP*/
-
 	/* Sizeof a line of bytes in the TIFF tile.
 	 */
 	if( wtiff->ready->Coding == VIPS_CODING_LABQ )
@@ -1489,7 +1537,7 @@ wtiff_pack2tiff( Wtiff *wtiff, Layer *layer,
 /* Write a set of tiles across the strip.
  */
 static int
-wtiff_layer_write_tile( Wtiff *wtiff, Layer *layer, VipsRegion *strip )
+wtiff_layer_write_tiles( Wtiff *wtiff, Layer *layer, VipsRegion *strip )
 {
 	VipsImage *im = layer->image;
 	VipsRect *area = &strip->valid;
@@ -1511,21 +1559,88 @@ wtiff_layer_write_tile( Wtiff *wtiff, Layer *layer, VipsRegion *strip )
 		tile.height = wtiff->tileh;
 		vips_rect_intersectrect( &tile, &image, &tile );
 
-		/* Have to repack pixels.
-		 */
-		wtiff_pack2tiff( wtiff, layer, strip, &tile, wtiff->tbuf );
-
 #ifdef DEBUG_VERBOSE
 		printf( "Writing %dx%d tile at position %dx%d to image %s\n",
 			tile.width, tile.height, tile.left, tile.top,
 			TIFFFileName( layer->tif ) );
 #endif /*DEBUG_VERBOSE*/
 
-		if( TIFFWriteTile( layer->tif, wtiff->tbuf, 
-			tile.left, tile.top, 0, 0 ) < 0 ) {
-			vips_error( "vips2tiff", 
-				"%s", _( "TIFF write tile failed" ) );
-			return( -1 );
+		if( wtiff->we_compress ) {
+			ttile_t tile_no = TIFFComputeTile( layer->tif,
+				tile.left, tile.top, 0, 0 );
+
+			VipsTarget *target;
+			int result;
+			unsigned char *buffer;
+			size_t length;
+
+			target = vips_target_new_to_memory();
+
+			switch( wtiff->compression ) {
+			case JP2K_LOSSY:
+				/* Sadly chroma subsample seems not to work
+				 * for edge tiles in tiff with jp2k
+				 * compression, so we always pass FALSE
+				 * instead of:
+				 *
+				 * 	!wtiff->rgbjpeg && wtiff->Q < 90,
+				 *
+				 * I've verified that the libvips jp2k
+				 * encode and decode subsample operations fill
+				 * the comps[i].data arrays correctly, so it
+				 * seems to be a openjpeg bug.
+				 *
+				 * FIXME ... try again with openjpeg 2.5,
+				 * when that comes.
+				 */
+				result = vips__foreign_load_jp2k_compress( 
+					strip, &tile, target,
+					wtiff->tilew, wtiff->tileh,
+					!wtiff->rgbjpeg,
+				 	// !wtiff->rgbjpeg && wtiff->Q < 90,
+					FALSE,
+					wtiff->lossless, 
+					wtiff->Q );
+				break;
+
+			default:
+				result = -1;
+				g_assert_not_reached();
+				break;
+			}
+
+			if( result ) {
+				g_object_unref( target );
+				return( -1 );
+			}
+
+			buffer = vips_target_steal( target, &length );
+
+			g_object_unref( target );
+
+			result = TIFFWriteRawTile( layer->tif, tile_no, 
+				buffer, length );
+
+			g_free( buffer );
+		
+			if( result < 0 ) {
+				vips_error( "vips2tiff", 
+					"%s", _( "TIFF write tile failed" ) );
+				return( -1 );
+			}
+		}
+		else {
+			/* Have to repack pixels for libtiff.
+			 */
+			wtiff_pack2tiff( wtiff, 
+				layer, strip, &tile, wtiff->tbuf );
+
+			if( TIFFWriteTile( layer->tif, wtiff->tbuf, 
+				tile.left, tile.top, 0, 0 ) < 0 ) {
+				vips_error( "vips2tiff", 
+					"%s", _( "TIFF write tile failed" ) );
+				return( -1 );
+			}
 		}
 	}
 
@@ -1672,7 +1787,7 @@ layer_strip_arrived( Layer *layer )
 	VipsRect image_area;
 
 	if( wtiff->tile ) 
-		result = wtiff_layer_write_tile( wtiff, layer, layer->strip );
+		result = wtiff_layer_write_tiles( wtiff, layer, layer->strip );
 	else
 		result = wtiff_layer_write_strip( wtiff, layer, layer->strip );
 	if( result )
@@ -1792,6 +1907,62 @@ write_strip( VipsRegion *region, VipsRect *area, void *a )
 #define CopyField( tag, v ) \
 	if( TIFFGetField( in, tag, &v ) ) TIFFSetField( out, tag, v )
 
+static int
+wtiff_copy_tiles( Wtiff *wtiff, TIFF *out, TIFF *in )
+{
+	const ttile_t n_tiles = TIFFNumberOfTiles( in );
+
+	tsize_t tile_size;
+	tdata_t buf;
+	ttile_t i;
+
+	if( wtiff->compression == COMPRESSION_JPEG ) 
+		tile_size = TIFFTileSize( in );
+	else 
+		/* If we will be copying raw tiles we need a buffer large 
+		 * enough to hold the largest compressed tile in any page.
+		 *
+		 * Allocate a buffer 2x the uncompressed tile size ... much 
+		 * simpler than searching every page for the largest tile with
+		 * TIFFTAG_TILEBYTECOUNTS.
+		 */
+		tile_size = 2 * wtiff->tls * wtiff->tileh;
+	buf = vips_malloc( NULL, tile_size );
+
+	for( i = 0; i < n_tiles; i++ ) {
+		tsize_t len;
+
+		/* If this is a JPEG-compressed TIFF, we need to decompress 
+		 * and recompress, since tiles are actually written in several 
+		 * places (coefficients go in the tile, huffman tables go 
+		 * elsewhere).
+		 *
+		 * For all other compression types, we can just use 
+		 * TIFFReadRawTile()/TIFFWriteRawTile().
+		 */
+		if( wtiff->compression == COMPRESSION_JPEG ) {
+			len = TIFFReadEncodedTile( in, i, buf, tile_size );
+			if( len <= 0 ||
+				TIFFWriteEncodedTile( out, i, buf, len ) < 0 ) {
+				g_free( buf );
+				return( -1 );
+			}
+		}
+		else {
+			len = TIFFReadRawTile( in, i, buf, tile_size );
+			if( len <= 0 ||
+				TIFFWriteRawTile( out, i, buf, len ) < 0 ) {
+				g_free( buf );
+				return( -1 );
+			}
+		}
+	}
+
+	g_free( buf );
+
+	return( 0 );
+}
+
 /* Copy a TIFF file ... we know we wrote it, so just copy the tags we know 
  * we might have set.
  */
@@ -1802,9 +1973,6 @@ wtiff_copy_tiff( Wtiff *wtiff, TIFF *out, TIFF *in )
 	uint16 ui16;
 	uint16 ui16_2;
 	float f;
-	tdata_t buf;
-	ttile_t tile;
-	ttile_t n;
 	uint16 *a;
 
 	/* All the fields we might have set.
@@ -1889,23 +2057,8 @@ wtiff_copy_tiff( Wtiff *wtiff, TIFF *out, TIFF *in )
 			wtiff_embed_imagedescription( wtiff, out ) )
 			return( -1 );
 
-	buf = vips_malloc( NULL, TIFFTileSize( in ) );
-	n = TIFFNumberOfTiles( in );
-	for( tile = 0; tile < n; tile++ ) {
-		tsize_t len;
-
-		/* It'd be good to use TIFFReadRawTile()/TIFFWtiffRawTile() 
-		 * here to save compression/decompression, but sadly it seems
-		 * not to work :-( investigate at some point.
-		 */
-		len = TIFFReadEncodedTile( in, tile, buf, -1 );
-		if( len < 0 ||
-			TIFFWriteEncodedTile( out, tile, buf, len ) < 0 ) {
-			g_free( buf );
-			return( -1 );
-		}
-	}
-	g_free( buf );
+	if( wtiff_copy_tiles( wtiff, out, in ) )
+		return( -1 );
 
 	return( 0 );
 }
@@ -2085,7 +2238,8 @@ vips__tiff_write( VipsImage *input, const char *filename,
 	int level, 
 	gboolean lossless,
 	VipsForeignDzDepth depth,
-	gboolean subifd )
+	gboolean subifd,
+	gboolean premultiply )
 {
 	Wtiff *wtiff;
 
@@ -2100,7 +2254,7 @@ vips__tiff_write( VipsImage *input, const char *filename,
                 tile, tile_width, tile_height, pyramid, bitdepth,
 		miniswhite, resunit, xres, yres, bigtiff, rgbjpeg, 
 		properties, strip, region_shrink, level, lossless, depth,
-		subifd )) )
+		subifd, premultiply )) )
 		return( -1 );
 
 	if( wtiff_write_image( wtiff ) ) { 
@@ -2131,7 +2285,8 @@ vips__tiff_write_buf( VipsImage *input,
 	int level, 
 	gboolean lossless,
 	VipsForeignDzDepth depth,
-	gboolean subifd )
+	gboolean subifd,
+	gboolean premultiply )
 {
 	Wtiff *wtiff;
 
@@ -2142,7 +2297,7 @@ vips__tiff_write_buf( VipsImage *input,
                 tile, tile_width, tile_height, pyramid, bitdepth,
 		miniswhite, resunit, xres, yres, bigtiff, rgbjpeg, 
 		properties, strip, region_shrink, level, lossless, depth,
-		subifd )) )
+		subifd, premultiply )) )
 		return( -1 );
 
 	wtiff->obuf = obuf;
