@@ -65,10 +65,10 @@
 #include <vips/thread.h>
 #include <vips/debug.h>
 
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 #include <windows.h>
 #include <lmerr.h>
-#endif /*OS_WIN32*/
+#endif /*G_OS_WIN32*/
 
 /**
  * SECTION: errors
@@ -200,7 +200,7 @@ vips_error_buffer_copy( void )
 
 	g_mutex_lock( vips__global_lock );
 	msg = g_strdup( vips_buf_all( &vips_error_buf ) );
-	vips_error_clear();
+	vips_buf_rewind( &vips_error_buf );
 	g_mutex_unlock( vips__global_lock );
 
 	return( msg );
@@ -294,7 +294,7 @@ vips_verror_system( int err, const char *domain, const char *fmt, va_list ap )
 {
 	vips_verror( domain, fmt, ap );
 
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 {
 	char *buf;
 
@@ -310,7 +310,7 @@ vips_verror_system( int err, const char *domain, const char *fmt, va_list ap )
 		LocalFree( buf );
 	}
 }
-#else /*OS_WIN32*/
+#else /*!G_OS_WIN32*/
 {
 	char *buf;
 
@@ -318,7 +318,7 @@ vips_verror_system( int err, const char *domain, const char *fmt, va_list ap )
 	vips_error( _( "unix error" ), "%s", buf );
 	g_free( buf );
 }
-#endif /*OS_WIN32*/
+#endif /*G_OS_WIN32*/
 }
 
 /**
@@ -430,7 +430,7 @@ vips_error_exit( const char *fmt, ... )
 	if( fmt ) {
 		va_list ap;
 
-		fprintf( stderr, "%s: ", g_get_prgname() );
+		fprintf( stderr, "%s: ", vips_get_prgname() );
 
 		va_start( ap, fmt );
 		(void) vfprintf( stderr, fmt, ap );
