@@ -70,6 +70,7 @@
 
 #include <vips/vips.h>
 #include <vips/thread.h>
+#include <vips/internal.h>
 #include <vips/debug.h>
 
 #ifdef VIPS_DEBUG_AMBER
@@ -106,8 +107,12 @@ typedef struct _Render {
 	 * threads. We can't easily use the gobject ref count system since we
 	 * need a lock around operations.
 	 */
+#if GLIB_CHECK_VERSION( 2, 58, 0 )
+	gatomicrefcount ref_count;
+#else
 	int ref_count;
-	GMutex *ref_count_lock;	
+	GMutex *ref_count_lock;
+#endif
 
 	/* Parameters.
 	 */
@@ -221,7 +226,11 @@ render_free( Render *render )
 {
 	VIPS_DEBUG_MSG_AMBER( "render_free: %p\n", render );
 
+#if GLIB_CHECK_VERSION( 2, 58, 0 )
+	g_assert ( g_atomic_ref_count_compare( &render->ref_count, 0 ) );
+#else
 	g_assert( render->ref_count == 0 );
+#endif
 
 	g_mutex_lock( render_dirty_lock );
 	if( g_slist_find( render_dirty_all, render ) ) {
@@ -234,7 +243,9 @@ render_free( Render *render )
 	}
 	g_mutex_unlock( render_dirty_lock );
 
+#if !GLIB_CHECK_VERSION( 2, 58, 0 )
 	vips_g_mutex_free( render->ref_count_lock );
+#endif
 	vips_g_mutex_free( render->lock );
 
 	vips_slist_map2( render->all, (VipsSListMap2Fn) tile_free, NULL, NULL );
@@ -259,10 +270,15 @@ render_free( Render *render )
 static int
 render_ref( Render *render )
 {
+#if GLIB_CHECK_VERSION( 2, 58, 0 )
+	g_assert( !g_atomic_ref_count_compare( &render->ref_count, 0 ) );
+	g_atomic_ref_count_inc( &render->ref_count );
+#else
 	g_mutex_lock( render->ref_count_lock );
 	g_assert( render->ref_count != 0 );
 	render->ref_count += 1;
 	g_mutex_unlock( render->ref_count_lock );
+#endif
 
 	return( 0 );
 }
@@ -272,11 +288,16 @@ render_unref( Render *render )
 {
 	int kill;
 
+#if GLIB_CHECK_VERSION( 2, 58, 0 )
+	g_assert( !g_atomic_ref_count_compare( &render->ref_count, 0 ) );
+	kill = g_atomic_ref_count_dec( &render->ref_count );
+#else
 	g_mutex_lock( render->ref_count_lock );
 	g_assert( render->ref_count > 0 );
 	render->ref_count -= 1;
 	kill = render->ref_count == 0;
 	g_mutex_unlock( render->ref_count_lock );
+#endif
 
 	if( kill )
 		render_free( render );
@@ -405,17 +426,6 @@ render_work( VipsThreadState *state, void *a )
 	}
 	tile->painted = TRUE;
 
-	/* All downstream images must drop caches, since we've (effectively)
-	 * modified render->out. 
-	 */
-	if( !render->shutdown ) 
-		vips_image_invalidate_all( render->out ); 
-	if( !render->shutdown &&
-		render->mask ) 
-		vips_image_invalidate_all( render->mask ); 
-
-	/* Now clients can update.
-	 */
 	if( !render->shutdown &&
 		render->notify ) 
 		render->notify( render->out, &tile->area, render->a );
@@ -547,8 +557,12 @@ render_new( VipsImage *in, VipsImage *out, VipsImage *mask,
 	 */
 	g_object_ref( in );
 
+#if GLIB_CHECK_VERSION( 2, 58, 0 )
+	g_atomic_ref_count_init( &render->ref_count );
+#else
 	render->ref_count = 1;
 	render->ref_count_lock = vips_g_mutex_new();
+#endif
 
 	render->in = in;
 	render->out = out;
@@ -985,7 +999,7 @@ render_dirty_get( void )
 	return( render );
 }
 
-/* Loop for the background render mananger thread.
+/* Loop for the background render manager thread.
  */
 static void *
 render_thread_main( void *client )
@@ -1031,13 +1045,18 @@ render_thread_main( void *client )
 }
 
 static void *
-vips__sink_screen_init( void *data )
+vips__sink_screen_once( void *data )
 {
 	g_assert( !render_thread ); 
 	g_assert( !render_dirty_lock ); 
 
 	render_dirty_lock = vips_g_mutex_new();
 	vips_semaphore_init( &n_render_dirty_sem, 0, "n_render_dirty" );
+
+	/* Don't use vips__thread_execute() since this thread will only be
+	 * ended by _shutdown, and that isn't always called early enough on
+	 * windows from atexit().
+	 */
 	render_thread = vips_g_thread_new( "sink_screen",
 		render_thread_main, NULL );
 
@@ -1105,7 +1124,7 @@ vips_sink_screen( VipsImage *in, VipsImage *out, VipsImage *mask,
 
 	Render *render;
 
-	VIPS_ONCE( &once, vips__sink_screen_init, NULL );
+	VIPS_ONCE( &once, vips__sink_screen_once, NULL );
 
 	if( tile_width <= 0 || tile_height <= 0 || 
 		max_tiles < -1 ) {
@@ -1146,23 +1165,29 @@ vips_sink_screen( VipsImage *in, VipsImage *out, VipsImage *mask,
 	return( 0 );
 }
 
-void
+int
 vips__print_renders( void )
 {
+	int n_leaks;
+
+	n_leaks = 0;
+
 #ifdef VIPS_DEBUG_AMBER
-	if( render_num_renders > 0 )
+	if( render_num_renders > 0 ) {
 		printf( "%d active renders\n", render_num_renders );
+		n_leaks += render_num_renders;
+	}
 #endif /*VIPS_DEBUG_AMBER*/
 
 	if( render_dirty_lock ) { 
-		int n_dirty;
-
 		g_mutex_lock( render_dirty_lock );
 
-		n_dirty = g_slist_length( render_dirty_all );
-		if( n_dirty > 0 ) 
-			printf( "%d dirty renders\n", n_dirty );
+		n_leaks += g_slist_length( render_dirty_all );
+		if( render_dirty_all ) 
+			printf( "dirty renders\n" );
 
 		g_mutex_unlock( render_dirty_lock );
 	}
+
+	return( n_leaks );
 }
