@@ -65,23 +65,24 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#ifdef OS_WIN32
-#include <io.h>
-#endif /*OS_WIN32*/
 
 #include <vips/vips.h>
 #include <vips/internal.h>
 #include <vips/debug.h>
 
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif /*G_OS_WIN32*/
+
 /* Try to make an O_BINARY ... sometimes need the leading '_'.
  */
-#ifdef BINARY_OPEN
+#if defined(G_PLATFORM_WIN32) || defined(G_WITH_CYGWIN)
 #ifndef O_BINARY
 #ifdef _O_BINARY
 #define O_BINARY _O_BINARY
 #endif /*_O_BINARY*/
 #endif /*!O_BINARY*/
-#endif /*BINARY_OPEN*/
+#endif /*defined(G_PLATFORM_WIN32) || defined(G_WITH_CYGWIN)*/
 
 /* If we have O_BINARY, add it to a mode flags set.
  */
@@ -307,12 +308,12 @@ vips_source_build( VipsObject *object )
 		connection->descriptor = dup( connection->descriptor );
 		connection->close_descriptor = connection->descriptor;
 
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 		/* Windows will create eg. stdin and stdout in text mode.
 		 * We always read in binary mode.
 		 */
 		_setmode( connection->descriptor, _O_BINARY );
-#endif /*OS_WIN32*/
+#endif /*G_OS_WIN32*/
 	}
 
 	if( vips_object_argument_isset( object, "blob" ) ) {
@@ -354,7 +355,8 @@ vips_source_seek_real( VipsSource *source, gint64 offset, int whence )
 	 * vips__seek() wrapper so we can seek long files on Windows.
 	 */
 	if( connection->descriptor != -1 )
-		return( vips__seek_no_error( connection->descriptor, offset, whence ) );
+		return( vips__seek_no_error( connection->descriptor, 
+			offset, whence ) );
 
 	return( -1 );
 }
@@ -563,8 +565,6 @@ vips_source_new_from_options( const char *options )
  *
  * Loaders should call this in response to the minimise signal on their output
  * image.
- *
- * Returns: 0 on success, or -1 on error.
  */
 void
 vips_source_minimise( VipsSource *source )
@@ -1158,8 +1158,10 @@ vips_source_seek( VipsSource *source, gint64 offset, int whence )
  * vips_source_rewind:
  * @source: source to operate on
  *
- * Rewind the source to the start. You can't do this after pixel decode phase
- * starts.
+ * Rewind the source to the start. 
+ *
+ * You can't always do this after the pixel decode phase starts -- for
+ * example, pipe-like sources can't be rewound.
  *
  * Returns: 0 on success, or -1 on error.
  */
@@ -1174,6 +1176,12 @@ vips_source_rewind( VipsSource *source )
 		vips_source_seek( source, 0, SEEK_SET ) != 0 )
 		return( -1 );
 
+	/* Back into sniff + header decode state.
+	 */
+	source->decode = FALSE;
+	if( !source->sniff )
+		source->sniff = g_byte_array_new();
+
 	SANITY( source );
 
 	return( 0 );
