@@ -41,6 +41,7 @@ DICOM_FILE = os.path.join(IMAGES, "dicom_test_image.dcm")
 BMP_FILE = os.path.join(IMAGES, "MARBLES.BMP")
 NIFTI_FILE = os.path.join(IMAGES, "avg152T1_LR_nifti.nii.gz")
 ICO_FILE = os.path.join(IMAGES, "favicon.ico")
+AVIF_FILE = os.path.join(IMAGES, "avif-orientation-6.avif")
 HEIC_FILE = os.path.join(IMAGES, "heic-orientation-6.heic")
 RGBA_FILE = os.path.join(IMAGES, "rgba.png")
 RGBA_CORRECT_FILE = os.path.join(IMAGES, "rgba-correct.ppm")
@@ -55,6 +56,7 @@ MOSAIC_MARKS = [[489, 140], [66, 141],
 MOSAIC_VERTICAL_MARKS = [[388, 44], [364, 346],
                          [384, 17], [385, 629],
                          [527, 42], [503, 959]]
+JP2K_FILE = os.path.join(IMAGES, "world.jp2")
 
 unsigned_formats = [pyvips.BandFormat.UCHAR,
                     pyvips.BandFormat.USHORT,
