@@ -138,12 +138,20 @@ main( int argc, char **argv )
 	textdomain( GETTEXT_PACKAGE );
 	setlocale( LC_ALL, "" );
 
+{
+	char *basename;
+
+	basename = g_path_get_basename( argv[0] );
+	g_set_prgname( basename );
+	g_free( basename );
+}
+
 	/* On Windows, argv is ascii-only .. use this to get a utf-8 version of
 	 * the args.
 	 */
-#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+#ifdef G_OS_WIN32
 	argv = g_win32_get_command_line();
-#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+#endif /*G_OS_WIN32*/
 
 	context = g_option_context_new( 
 		_( "vipsedit - edit vips file header" ) );
@@ -153,11 +161,11 @@ main( int argc, char **argv )
 	g_option_group_set_translation_domain( main_group, GETTEXT_PACKAGE );
 	g_option_context_set_main_group( context, main_group );
 
-#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+#ifdef G_OS_WIN32
 	if( !g_option_context_parse_strv( context, &argv, &error ) ) 
-#else /*!HAVE_G_WIN32_GET_COMMAND_LINE*/
+#else /*!G_OS_WIN32*/
 	if( !g_option_context_parse( context, &argc, &argv, &error ) ) 
-#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+#endif /*G_OS_WIN32*/
 	{
 		vips_g_error( &error );
 
@@ -173,6 +181,7 @@ main( int argc, char **argv )
 	if( argc != 2 ) { 
 		fprintf( stderr, _( "usage: %s [OPTION...] vips-file\n" ), 
 			g_get_prgname() );
+
 		exit( -1 );
 	}
 
@@ -269,11 +278,11 @@ main( int argc, char **argv )
 
 	g_object_unref( im );
 
-	/* We don't free this on error exit, sadly.
-	 */
-#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+	g_option_context_free( context );
+
+#ifdef G_OS_WIN32
 	g_strfreev( argv ); 
-#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+#endif /*G_OS_WIN32*/
 
 	vips_shutdown();
 
